/////////////////////////////////////////////////////////////////////////
////                                                                 ////
////                        ex_usb_serial.c  
//                StdDaq main code, adapted from above               ////
////                                                                 ////
//// A demonstration of the USB CDC API that is provided by CCS.     ////
//// The USB CDC API that CCS provides will create a virtual UART    ////
//// port.  USB CDC drivers are included with most versions of       ////
//// Microsoft Windows, and when properly loaded will create a COMx  ////
//// port from which you can write and read to your PIC device       ////
//// like any serial device that has a COMx port.                    ////
////                                                                 ////
////                                                                 ////
//// See usb_cdc.h for API documentation.                            ////
////                                                                 ////
//// This file is part of CCS's PIC USB driver code.  See USB.H      ////
//// for more documentation and a list of examples.                  ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////                                                                 ////
//// VERSION HISTORY                                                 ////
////                                                                 ////
//// Oct 15th, 2010:                                                 ////
////  Potential bug in CDC that causes lost PIC->PC chars resolved?  ////
////  Added initial 18F47J53 family support.                         ////
////  Added USB_ISR_POLLING support.  Define this and interrupts     ////
////     will not be used.  usb_task() must be called periodically   ////
////     in your main loop.  If it is not called faster than once    ////
////     per millisecond, USB may not work (PIC18 and PIC24 only).   ////
////                                                                 ////
//// April 28th, 2010:                                               ////
////  Added ex_usb_common.h.                                         ////
////  Initial support for CCS PIC24USB board.                        ////
////                                                                 ////
//// March 5th, 2009:                                                ////
////   Cleanup for Wizard.                                           ////
////   PIC24 Initial release.                                        ////
////                                                                 ////
//// July 1st, 2005: Initial Release.                                ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,2005 Custom Computer Services         ////
//// This source code may only be used by licensed users of the CCS  ////
//// C compiler.  This source code may only be distributed to other  ////
//// licensed users of the CCS C compiler.  No other use,            ////
//// reproduction or distribution is permitted without written       ////
//// permission.  Derivative programs created using this software    ////
//// in object code form are not restricted in any way.              ////
////                                                                 ////  
// So totally changed by me, Doug Coulter, I kind of doubt the (C)   ////
//applies up there anymore.  My parts (C) 2011 Doug Coulter          ////
/////////////////////////////////////////////////////////////////////////


/*  version history for std counter project - (C)2011 Doug Coulter
removed USB connection sensing code, as this can't run without USB power
removed a bunch of junk for other CPU's and other USB hardware


Added opsysutils.h to contain the project specific code in its own file
separate from the pure USB driver/example stuff
(start of project)
Pulled all the CCS files to project directory, and am making sure to change
the includes to "" instead of <> so it will use the local copies.

*/

// ex_usb_common.h contains the #fuses and a few defines
#include "ex_usb_common.h" // changed example to "" so it will use local .h
// code for CDC interface (as opposed to HID or block xfers)
#include "usb_cdc.h" // "" to use local copy
// not sure this should be last, but that gives it the last word on IO setup
#include "opsysutils.h" // my added stuff for the real project

/////////////////////////////////////////////////////////////////////////////
//
// usb_debug_task()
//
// When called periodically, displays debugging information over serial
// to display enumeration and connection states.  Also lights LED1 based upon
// enumeration and status.
//
/////////////////////////////////////////////////////////////////////////////
/* not using this code anymore, but keeping it available for later
void usb_debug_task(void)
{
   static int8 last_connected;
   static int8 last_enumerated;
   int8 new_connected;
   int8 new_enumerated;
   static int8 last_cdc;
   int8 new_cdc;

   new_connected=usb_attached();
   new_enumerated=usb_enumerated();
   new_cdc=usb_cdc_connected();

   if (new_enumerated)
      LED_ON(LED1);
   else
      LED_OFF(LED1);

   if (new_cdc)
      LED_ON(LED2);
   else
      LED_OFF(LED2);

   if (usb_cdc_carrier.dte_present)
      LED_ON(LED3);
   else
      LED_OFF(LED3);

   if (new_connected && !last_connected)
      uart_printf("USB connected, waiting for enumeration...\r\n");
   if (!new_connected && last_connected)
      uart_printf("USB disconnected, waiting for connection...\r\n");
   if (new_enumerated && !last_enumerated)
      uart_printf("USB enumerated by PC/HOST\r\n\n");
   if (!new_enumerated && last_enumerated)
      uart_printf("USB unenumerated by PC/HOST, waiting for enumeration...\r\n");
   if (new_cdc && !last_cdc)
      uart_printf("Serial program initiated on USB<->UART COM Port\r\n\n");

   last_connected=new_connected; // z-1 signal processing, detect change
   last_enumerated=new_enumerated;
   last_cdc=new_cdc;
}
*/

void main(void) 
{
   char c;   
   opsysinit(); // Clab's stuff - setup timers, counters, interrupts, memory
//   LED_OFF(LED1); // leds not used on std counter proto yet
//   LED_OFF(LED2);
//   LED_OFF(LED3);

/* debug code for rs232   uart_printf("\r\nC-lab Standard Counter\r\n");
  #if defined(__PCH__)
   uart_printf("PCH: v");
   uart_printf(__PCH__);
  #endif
   uart_printf("\r\n");
*/   
   usb_init_cs(); // initialize the USB library

/*************************************************************************
// we just go around this loop till power fails.  Call each task in turn,
// and design the tasks to return quick -- no blocking or the whole thing
// will hang.  Check for work, do a little, return, you'll get called
// again soon if everyone follows the rules.
*/////////////////////////////////////////////////////////////////////////
   while (TRUE) 
   {
      usb_task(); // attend to any USB activity -- connection stuff

//@@@ not needed now usb_debug_task(); // do leds showing USB connection status
// adding my stuff to this forever loop
      if (secflag) // timer 1 isr sets this flag once a second
      {
       TimeMachine(); // keep running time clock up to speed, print time
       DoCounts(); // print the rest of the output line 
       DoAD();    // output a/d results, restart etc
// do anthing else needing 1/second timing here
      } // end if seconds flag
      
      LedMachine(); // turn off comm led when its oneshot expires

      if (uart_kbhit()) // pass stuff between the 232 and usb ports
      { // leaving this in for debugging, but not really used
         c=uart_getc();// the if's below make this "windows safe" /r/n
         if (c=='\n') {usb_cdc_putc('\r'); usb_cdc_putc('\n');}
         else if (c=='\r') {usb_cdc_putc('\r'); usb_cdc_putc('\n');}
         else if (c=='!') {uart_printf(usb_cdc_putc,"HELLO-WORLD-DONE");}
         else {usb_cdc_putc(c);}
      }
      if (usb_cdc_kbhit()) 
      { // handle dumb windows that needs two chars for a new line
         c=usb_cdc_getc();
         if (c=='\n') {uart_putc('\r'); uart_putc('\n');}
         else if (c=='\r') {uart_putc('\r'); uart_putc('\n');}
         else if (c=='!') ClearTimeCounts(); // reset time and counters
         
         else {uart_putc(c);}
      }
   } // end while true
} // end main
