/*
 Some utilities from my old opsys for use with USB




*/

// declare opsys globals here
// CCS default is unsigned, but it pays to be explicit for the humans
int8 i;  // I use this over and over, with care about nesting
int8 ii; // iterator for interrupts (only) to use

unsigned int8 secdiv; // tens of milliseconds in this version (0-99)
unsigned int8 seconds; // the time since startup
unsigned int8 minutes;
unsigned int16 hours;
int1 secflag; // boolean set once a second by Timer1 interrupt
///////////////////
unsigned int16 CountsHigh; // high part of timer0 total count
unsigned int16 Counts3High; // high part of timer3 total count

#define AD_CHANNELS 4
unsigned int8 adi; // a/d index
unsigned int8 adcnt; // number of samples taken so far

unsigned int16 ADRes[AD_CHANNELS]; // accumulated a/d results

unsigned int32 CountTemp; // for subtraction
unsigned int32 LastTotal; // created at runtime as needed for subtract
unsigned int32 Last3Total; //other counter, to calc current second count
unsigned int32 CurrentCount; //@@@ 65k counts per second - software limit
unsigned int8 LedCount;    // for display
//////////////////
#define NO_ONESHOTS 8 // how many oneshots we'll handle
unsigned int8 oneshots[NO_ONESHOTS]; // the oneshots, globals
//////////////////////////////

char outbuf[64]; // where we hold what we're going to send to the host

///////////////////////////////////////////////////////////////////////////

void ClearTimeCounts()
// clears times and counts.  Used at init or a runtime reset from PC
{
secdiv = seconds = minutes = 0; // runtime clock, secdivs are 10 ms
CountsHigh = Counts3High = hours = 0; //grouped by size of integer
secflag = FALSE; // gets set once a second by timer1 isr.  A bool
LastTotal = Last3Total = 0; // init 32 bit integer
for (i=0;i<NO_ONESHOTS;i++) oneshots[i] = 0; // zero the oneshots
set_timer0(0); // clear out timer hardware too
set_timer3(0); // both of em
}
///////////////////////////////////////////////////////////////////////////
void ClearAD()
{ // I believe in copius comments, but really?
 for (i=0;i<AD_CHANNELS;i++) ADRes[i] = 0;
 adi = adcnt = 0; // clear out channel index and count of samples
}
////////////////////////////////////////////////////////////////////////////
void opsysinit()
{ // set things up

 ClearTimeCounts();  // clear timer and counters
 ClearAD();
/*************** Hardware init *************************/

// timer one counts the cpu system clock, to interrupt at 100hz (10ms)
// this is what we use for runtime and oneshot timer clock
setup_timer_1(T1_INTERNAL | T1_DIV_BY_8); // 1.5 mhz with 12 mhz cycle rate
set_timer1(65536-15000); // set to overflow at 100 hz
enable_interrupts(INT_TIMER1);

// timer 0 is the (first) counter we're counting -- geiger pancake
setup_timer_0(T0_EXT_H_TO_L | T0_DIV_1); // divides by two unless specified
set_timer0(0); // clear the timer
set_tris_a(get_tris_a() | 0x10); // make sure it's an input
enable_interrupts(INT_TIMER0);

// added later, a second counter we'll count inputs on -- optional
setup_timer_3 (T3_EXTERNAL | T3_DIV_BY_1);
set_timer3(0);
set_tris_c(get_tris_c() | 0x01); // make an input pin for sure (dox unclear)
enable_interrupts(INT_TIMER3); // make it live

//setup_vref(VREF_HIGH | VREF_F5 | 6); // setup vref for output on RA2
//set_tris_a(get_tris_a() & 0b11011111); // make a5 output for comparator 2
//set_tris_a(get_tris_a() | 0b00000010); // make a1 input for comparator ?? a/d setup
//setup_comparator(A0_A3_A1_A2_OUT_ON_A4_A5); // we'll only use C2 here

// a/d setup things here
 setup_adc_ports(AN0_TO_AN3); // select first 4 channels as analog
 setup_adc(ADC_CLOCK_DIV_64); // set a/d clock
 ADCON2 |= ADTAD8;            // setup auto-delay for sample time
 enable_interrupts(INT_AD);   // interrupt-driven, that's us 

// their code enables interrupts "somewhere" -- but make sure
enable_interrupts(GLOBAL); // turn on all enabled interrupts
}
/////////////////////////////////////////////////////////////////////////////

// subroutines

///////////////////////////////////////
// Call this once a second from the main loop.  Clears the seconds flag and
// does the mundane conversions for time in seconds to clock time (since start)
// Timer1 interrupt sets secflag for background to know when to call us
void TimeMachine()
{
 if (++seconds >= 60) // the use of >= makes this work even without good init
 {
  seconds = 0; // new minute
  if (++minutes >= 60) // >= belt AND suspenders
  {
   minutes = 0; // new hour
   hours++; // so do that increment too
  } // end if minutes overflow
 }// end if seconds overflow
// we just hope hours don't overflow in a 16 bit int -- that's a long run
 secflag = FALSE; // clear seconds flag, set by timer1 interrupt

 
  // print first part of output line - both serial and USB output
  sprintf(outbuf,"\n%Ld:%02u:%02u ",hours,minutes,seconds); // trailing space
  uart_printf("%s",outbuf); // side benefit - blocking call lets USB get ready
  usb_cdc_puts(outbuf); // for this, so we're using uart as a delay...
}
///////////////////////////////////////
void DoCounts()
{ // called once per second (we hope!)
 CountTemp = make32(CountsHigh,get_timer0()); // get total counts
 CurrentCount = CountTemp - LastTotal; // counts for this second
//@@@ if the above returns a negative number, we should fix the rollover error
// and maybe have to do this in all 32 bit to even tell....
  output_high (PIN_D7); // turn on comm led
// compute how long to flash the led
 if (95 < CurrentCount) LedCount = 95;
 else LedCount = CurrentCount;
 oneshots[0] = 1 + LedCount; // leave on for 1/100 second + counts/sec
 
 
  //output to uart (used as delay/debug) and USB both here
  sprintf(outbuf,"C0 %Lu TC0 %Lu",CurrentCount,CountTemp);
  uart_printf("%s",outbuf); //output to serial port
  usb_cdc_puts(outbuf); // for this, so we're using uart as a delay...
  LastTotal = CountTemp; // keep up to speed on this
  
 // now the code for the other counter, which can share *some* of the variables
  CountTemp = make32(Counts3High,get_timer3()); // get total for C3
  CurrentCount = CountTemp - Last3Total; //same as above, but one unique var

  sprintf(outbuf," C1 %Lu TC1 %Lu",CurrentCount,CountTemp);
  uart_printf("%s",outbuf); // serial port and delay
  usb_cdc_puts(outbuf); // to USB too
  Last3Total = CountTemp; // keep track of last raw total number
}

///////////////////////////////////////
void DoAD()
{ // output existing a/d channel data and restart collection
 for (i=0;i<AD_CHANNELS;i++)
 {
  sprintf(outbuf," AD%u %Lu",i,ADRes[i]);
  uart_printf ("%s",outbuf);
  usb_cdc_puts(outbuf);
 }
 ClearAD(); // remove old summations
 // now start next aq
 set_adc_channel(adi); // start at channel zero again
 read_adc(ADC_START_ONLY); // start it, we'll get an interrupt later

}

///////////////////////////////////////
void LedMachine()
{ // control the comm led.  Once a second, flash it.  Set duration with
// a oneshot related to the current count total
// this is where we turn it back off when the oneshot expires
 if (0 == oneshots[0]) output_low(PIN_D7);
}



///////////////////////////////////////



/******************Interrupts*************************************************/
///////////////////////////////////////
#INT_TIMER1
void intTimer1()
{
 set_timer1(65536-15000); // get ready for next time, count for 10ms
 secdiv++; // count 10 ms times till a second's worth
 if (secdiv >= 100)
  {
   secdiv = 0; // ready for next second's counting
   secflag = TRUE; // tell background about the new second
  }
// oneshots code here...count down any non-zero oneshots till they are zero
// at 10 ms per count.  You can set these in the background, then check for
// done once per main loop, and your timing delay is what you set.  This way
// you don't have to sit in a spin loop and wait, so other things can be
// run while a oneshot counts down
 for (ii=0;ii<NO_ONESHOTS;ii++)
 {
  if (oneshots[ii]) oneshots[ii]--; // count down any non-zero ones
 }
}
//////////////////////////////////////////
//  This counts another 16 bits when timer0 overflows for 32 bits total
//  low part of count is timer0 itself
#INT_TIMER0
void intTimer0()
{ // not a real complex ISR, just count the high 16 bits on overflow
 ++CountsHigh; // keep track of overflows
}
/////////////////////////////////////////////////////////////////
#INT_TIMER3
void intTimer3 ()
{ // count alternate input pin too
 ++Counts3High; // same as above for another counter
}
/////////////////////////////////////////////////////////////////
#INT_AD
void intAD()
// we will take 64 samples for each channel, and sum the results into
// the array ADRes[AD_CHANNELS].  This is better than just getting each
// channle once, and gives a 16 bit output from a 10 bit a/d, but...it's
// not really good to 16 bits - maybe 12?  Smooths out some noise...
// adcnt is how many samples, adi is current a/d channel
// since we can get the 256 total samples this implies for 64 per in a lot
// less than the 1 second we have, no done flag is needed for background
{
 AdRes[adi++] += read_adc(ADC_READ_ONLY); // grab the converted sample and sum
 
 if (adi > (AD_CHANNELS - 1)) // prepare to get next channel sample
 {
  adi = 0;  // back to channel zero
  adcnt++; // count a full roundy round
 }
 if (64 > adcnt) // we're still busy, so start again
 {
  set_adc_channel(adi); // set the input mux to next channel
  read_adc(ADC_START_ONLY); // start it, we'll get an interrupt later
 } // else, just don't start another conversion 
}
